require "studio_game/player"

module StudioGame
  describe Player do

    before do
      $stdout = StringIO.new
      @health = 100
      @player = Player.new("larry", @health)
    end

    it "has capitalized player name" do
      expect(@player.name).to be == "Larry"
    end

    it "can be created from a CSV string" do
      player = Player.from_csv("larry,150")
      expect(player.name) == "Larry"
      expect(player.health) == 150
    end

    it "has an ititial health" do
      expect(@player.health).to be == 100 
    end

    it "has a string representation" do
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:hammer, 50))
      expect(@player.to_s).to be == "I'm Larry with health = 100, points = 100, and score = 200."
    end

    it "computes a score as the sum of its health and length of name" do
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:hammer, 50))
      expect(@player.score).to be == (200)
    end

    it "increases health by 15 when w00ted" do
      @player.w00t
      expect(@player.health) == (@health + 15)
    end

    it "decreases health by 10 when blammed" do
      @player.blam
      expect(@player.health) == (@health - 10)
    end

    it "computes points as the sum of all treasure points" do
      expect(@player.points) == 0

      @player.found_treasure(Treasure.new(:hammer, 50))

      expect(@player.points) == 50

      @player.found_treasure(Treasure.new(:crowbar, 400))

      expect(@player.points) == 450

      @player.found_treasure(Treasure.new(:hammer, 50))

      expect(@player.points) == 500
    end

    it "yields each found treasure and its total points" do
      @player.found_treasure(Treasure.new(:skillet, 100))
      @player.found_treasure(Treasure.new(:skillet, 100))
      @player.found_treasure(Treasure.new(:hammer, 50))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      @player.found_treasure(Treasure.new(:bottle, 5))
      yielded = []
      @player.each_found_treasure do |treasure|
        yielded << treasure
      end
      expect(yielded) == [
        Treasure.new(:skillet, 200),
        Treasure.new(:hammer, 50),
        Treasure.new(:bottle, 25)
     ]
    end

    context "initial health greater than 100" do
      before do
        @player = Player.new("larry", 150)
      end

      it "determines if player is strong" do
        expect(@player).to be_strong
      end
    end

      context "initial health 100 or less" do
      before do
        @player = Player.new("larry", 100)
      end

      it "determines if player is wimpy" do
        expect(@player).not_to be_strong
      end
    end

    context "in a collection of players" do
    before do
      @player1 = Player.new("moe", 100)
      @player2 = Player.new("larry", 200)
      @player3 = Player.new("curly", 300)

      @players = [@player1, @player2, @player3]
    end

    it "is sorted by decreasing score" do
      expect(@players.sort).to eq([@player3, @player2, @player1])
    end
  end

  end
end